# Define all tags for api
TAGS_METADATA = [
    {
        'name': 'greet',
        'description': 'Greeting API',
        'externalDocs': {
            'description': 'None',
            'url': 'https://external-docs'
        }
    },
    {
        'name': 'account',
        'description': 'Các API liên quan tài khoản người dùng',
        'externalDocs': {
            'description': 'None',
            'url': 'https://external-docs'
        }
    }
]
