#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['fastapi_project_pattern']

package_data = \
{'': ['*']}

install_requires = \
['GitPython >= 3.1.24']

setup(name='fastapi_project_pattern',
      version='1.1.8',
      description='This project was created as a shortcut for fastapi project creation',
      author=None,
      author_email='Le Tanh Sang <sang.tanhle@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
